// u_c_person
// 2013-02-09

// -- (C) John COLIBRI 2013
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_string_serializer;
  interface
    uses Classes, Rtti, System.TypInfo;

    type c_serializer=
             Class
               function f_serialize(p_c_object: tObject): String;
             End; // c_serializer

         c_deserializer=
             Class
               procedure deserialize(p_string: String; p_c_object: tObject);
             End; // c_deserializer

  implementation
    uses SysUtils, u_display_simple;

    const // -- to know those, look at tValue.ToString
          k_not_serializable= [tkUnknown,
              tkSet,
              tkClass, tkMethod,
              tkVariant, tkArray, tkRecord, tkInterface, tkDynArray,
              tkClassRef, tkPointer, tkProcedure];

          k_toquote_kind= [tkChar, tkString, tkWChar, tkLString, tkWString,
              tkUString];

    function c_serializer.f_serialize(p_c_object: tObject): String;

      function f_serialize_property(p_tvalue: tValue): String;
        begin
          if p_tvalue.IsEmpty
            then Result:= 'Nil'
            else Result:= p_tvalue.ToString;

          if p_tvalue.Kind in k_toquote_kind then
            Result:= QuotedStr(Result);
        end; // f_serialize_property

      var l_c_result_list: tStringList;
          l_c_object_type: tRttiType;
          l_rtti_context: tRttiContext;
          l_c_rtti_property: tRttiProperty;
          l_tvalue: tValue;

      begin // f_serialize
        if p_c_object= Nil
          then Exit('');

        display('> serialize '+ p_c_object.ClassName);
        l_c_result_list:= tStringList.Create;

        l_c_object_type:= l_rtti_context.GetType(p_c_object.ClassType);
        display('object_classtype '+ l_c_object_type.ClassName);

        for l_c_rtti_property in l_c_object_type.GetProperties do
        begin
          display('  prop '+ l_c_rtti_property.ClassName);
          if l_c_rtti_property.IsReadable and l_c_rtti_property.IsWritable
              and not (l_c_rtti_property.PropertyType.Handle.Kind in k_not_serializable)
            then begin
                l_tvalue:= l_c_rtti_property.GetValue(p_c_object);
                l_c_result_list.Values[l_c_rtti_property.Name]:= f_serialize_property(l_tvalue);
              end;
        end;

        Result:= l_c_result_list.Text;
        l_c_result_list.Free;
        display('< serialize ');
      end; // f_serialize

    // -- c_deserializer

    procedure c_deserializer.deserialize(p_string: String;
        p_c_object: tObject);

      function f_deserialize_property(p_value: String;
          p_typeinfo: pTypeInfo): tValue;
        begin
          case p_typeinfo.Kind of
            tkInteger: Result:= StrToInt(p_value);
            tkInt64: ;
            tkEnumeration: Result:= tValue.FromOrdinal(p_typeinfo, System.TypInfo.GetEnumValue(p_typeinfo, p_value));
            tkFloat: Result:= StrToFloat(p_value);

            tkChar, tkWChar, tkLString, tkWString, tkUString,
              tkString: Result:= p_value;

            tkSet: ;
            tkClass: ;
            tkMethod: ;
            tkVariant: ;
            tkArray: ;
            tkRecord: ;
            tkInterface: ;

  (*
            tkDynArray: ;
            tkClassRef: ;
            tkPointer: ;
            tkProcedure: ;
            tkUnknown: ;
  *)
          end;
        end; // f_deserialize_property

      var l_c_input_list: tStringList;
          l_rtti_context: tRttiContext;

          l_c_object_rtti_type: tRttiType;
          l_list_index: integer;
          l_c_rtti_property: tRttiProperty;
          l_property_name, l_property_value: String;
          l_tvalue: tValue;
          l_string: String;

      begin // deserialize
        display('> deserialize '+ p_c_object.ClassName);

        l_c_input_list:= tStringList.Create;
        l_c_input_list.Text:= p_string;

        l_c_object_rtti_type:= l_rtti_context.GetType(p_c_object.ClassType);

        for l_list_index:= 0 to l_c_input_list.Count- 1 do
        begin
          // -- "Caption='Ok'"
          // -- form the property name, get the tRttiProperty
          display('  prop_name '+ l_property_name);
          l_property_name:= l_c_input_list.Names[l_list_index];
          l_c_rtti_property:= l_c_object_rtti_type.GetProperty(l_property_name);

          // -- from the value, build a tValue
          if (Self= Nil) or (l_c_rtti_property.PropertyType.Handle.Kind in k_not_serializable)
            then Continue;

          // -- from the string value, get the tValue
          l_property_value:= l_c_input_list.ValueFromIndex[l_list_index];
          l_tvalue:= f_deserialize_property(l_property_value, l_c_rtti_property.PropertyType.Handle);

          // -- set the object property value
          if not l_tvalue.IsEmpty
            then begin
                display(l_tvalue.ToString);
                if l_c_rtti_property.PropertyType.Handle.Kind in k_toquote_kind
                  then begin
                      l_string:= l_tvalue.AsString;
                      l_c_rtti_property.SetValue(p_c_object, AnsiDequotedStr(l_string, ''''))

                    end
                  else l_c_rtti_property.SetValue(p_c_object, l_tvalue)
              end
            else display('value empty');
        end;
        l_c_input_list.Free;

        display('< deserialize ');
      end; // deserialize

end.
